@@ -37,8 +37,8 @@ def dict_wrapped_result(func):
     """Wraps a call to a function so that the return value is put into a
     single item dictionary, with the key 'result'.
     
-    @param func Function to wrap
-    @return a new function
+    :param func: Function to wrap
+    :return: a new function
     """
     def d_w_r(*args, **kw):
         return dict(result=func(*args, **kw))
@@ -53,8 +53,8 @@ def wsexpose(return_type=basestring):
     can be returned because many languages only support a single
     return value from methods.
     
-    @param return_type the class that will be returned. This is used to
-                       help statically typed clients."""
+    :param return_type: the class that will be returned. This is used to
+                        help statically typed clients."""
     def entangle(func):
         fi = register(func)
         fi.return_type = return_type
@@ -233,16 +233,17 @@ class WebServicesController(object):
         
     
 class WebServicesRoot(WebServicesController):
-    """The root of a multi-protocol web service."""
+    """The root of a multi-protocol web service.
+
+    :param baseURL: Sets the URL path for this controller (trailing '/' 
+                    included). Some protocols need to know this.
+    :param tns: The SOAP target namespace (defaults to baseURL+soap/)
+    :param typenamespace: the namespace for the SOAP types (defaults to
+                          baseURL+soap/types)
+    """
     
     def __init__(self, baseURL, tns=None, typenamespace=None):
         """Constructor for a WebServicesRoot controller.
-        
-        @param baseURL Sets the URL path for this controller. Some protocols
-            need to know this.
-        @param tns The SOAP target namespace (defaults to baseURL+soap/)
-        @param typenamespace the namespace for the SOAP types (defaults to
-            baseURL+soap/types)
         """
         
         if not tns:
