@@ -1,6 +1,6 @@
 # Release information about TGWebServices
 
-version = "1.2.3"
+version = "1.2.4"
 
 description = "Multiprotocol Web Services for TurboGears"
 long_description = """TurboGears gives you a plain HTTP with JSON return
@@ -35,6 +35,9 @@ an HTTP Accept: text/javascript header) and you'll get JSON back.
 The great thing about this is that the code above looks like a '''normal
 Python function''' and doesn't know a thing about web services.
 
+A more complete documentation can be found at
+http://wiki.tgws.googlecode.com/hg/index.html.
+
 Features
 --------
 
@@ -46,11 +49,12 @@ Features
   to generate conveniently usable interfaces
 * Can output instances of your own classes
 * Can also accept instances of your classes as input
-* Works with TurboGears 1.0
+* Works with TurboGears 1.0 and 1.1
 * MIT license allows for unrestricted use"""
-author = "Kevin Dangoor"
+author = "Kevin Dangoor, Christophe de Vienne"
 email = "turbogears-web-services@googlegroups.com"
-copyright = "Copyright 2006, 2007 Kevin Dangoor, Arbor Networks"
+copyright = "Copyright 2006, 2007 Kevin Dangoor, Arbor Networks. 2008-2010 The TGWebServices development team."
 
 url = "http://code.google.com/p/tgws/"
 license = "MIT"
+
