@@ -2,10 +2,16 @@
 
 import inspect
 import types
+import base64
 
 from turbojson import jsonify
 
+from datetime import datetime, date, time
+
 class typedproperty(property):
+    """Wraps the standard Python property function allowing you
+    to specify what return type will be coming from your property's
+    getter method."""
     def __init__(self, type, *args, **kw):
         self.type = type
         super(typedproperty, self).__init__(*args, **kw)
@@ -23,11 +29,40 @@ class unsigned(object):
             return int(value)
         else:
             return int()
+
+class binary(object):
+    """Represents base64-encoded binary datas to the outside world."""
+    def __init__(self, value=None):
+        self.data = value
         
+    def base64(self):
+        if self.data is not None:
+            return base64.encodestring(self.data)
+
+    def __json__(self):
+        return self.base64()
+
+    def __str__(self):
+        return self.data
+
 # These are the primitive types that are generally understood equally well
 # by the various protocols.
 primitives = set([int, basestring, str, unicode, float, bool, list, dict,
-                  long, typedproperty, unsigned])
+                  long, typedproperty, unsigned,
+                  datetime, date, time,
+                  binary])
+
+@jsonify.jsonify.when("isinstance(obj, datetime)")
+def jsonify_datetime(obj):
+    return obj.isoformat()
+
+@jsonify.jsonify.when("isinstance(obj, date)")
+def jsonify_date(obj):
+    return obj.isoformat()
+
+@jsonify.jsonify.when("isinstance(obj, time)")
+def jsonify_time(obj):
+    return obj.isoformat()
 
 class FunctionInfo(object):
     """Keeps track of information about a function that is available for
