@@ -7,6 +7,8 @@ import traceback
 import time
 import logging
 
+import datetime
+
 from peak.rules import abstract, when
 import cherrypy
 
@@ -20,7 +22,7 @@ from genshi.builder import tag, Element, Namespace
 from turbogears import controllers, expose as tgexpose, error_handler, \
                        exception_handler, validators
 
-from tgwebservices.runtime import primitives, ctvalues, typedproperty, unsigned
+from tgwebservices.runtime import primitives, ctvalues, typedproperty, unsigned, binary
 
 log = logging.getLogger("tgwebservices.soap")
 
@@ -53,7 +55,36 @@ def soapbool(name, return_type, value):
     elem(str(value).lower())
     return elem
 
-@when(soap_value, "type(value) not in primitives")
+@when(soap_value, "type(value) == datetime.date")
+def soapdate(name, return_type, value):
+    elem = Element(name, **{"xsi:type" : "xsd:date"})
+    elem(value.isoformat())
+    return elem
+
+@when(soap_value, "type(value) == datetime.time")
+def soaptime(name, return_type, value):
+    elem = Element(name, **{"xsi:type" : "xsd:time"})
+    elem(value.isoformat())
+    return elem
+
+@when(soap_value, "type(value) == datetime.datetime")
+def soapdatetime(name, return_type, value):
+    elem = Element(name, **{"xsi:type" : "xsd:dateTime"})
+    elem(value.isoformat())
+    return elem
+
+@when(soap_value, "type(value) == binary")
+def soapbinary(name, return_type, value):
+    elem = Element(name, **{"xsi:type" : "xsd:base64Binary"})
+    elem(value.base64())
+    return elem
+
+@when(soap_value, "value is None")
+def soapnil(name, return_type, value):
+    elem = Element(name, **{"xsi:nil": "true"})
+    return elem
+
+@when(soap_value, "value is not None and type(value) not in primitives")
 def soap_instance(name, return_type, value):
     """Handles an instance of a complex type."""
     elem = Element(name, **{"xsi:type" : value.__class__.__name__})
@@ -89,7 +120,11 @@ type_registry = {
     long : "xsd:long",
     float : "xsd:float",
     bool : "xsd:boolean",
-    unsigned : "xsd:unsignedInt"
+    unsigned : "xsd:unsignedInt",
+    datetime.datetime : "xsd:dateTime",
+    datetime.date : "xsd:date",
+    datetime.time : "xsd:time",
+    binary : "xsd:base64Binary"
 }
 
 # Maps python types to appropriately named array types.
@@ -230,7 +265,7 @@ class SoapController(controllers.Controller):
         return data
     
     @tgexpose("genshi:tgwebservices.templates.wsdl", format="xml",
-              content_type="text/xml", fragment=True)
+              content_type="text/xml")
     def api_wsdl(self, xmlsoap=False):
         """Produces the WSDL for this SOAP service."""
         if self.arrays is None:
@@ -265,6 +300,8 @@ class SoapController(controllers.Controller):
         arrays = set()
         for func in self.wscontroller._ws_funcs.values():
             soap_type(func._ws_func_info.return_type, arrays)
+            for type in func._ws_func_info.input_types.values():
+                soap_type(type, arrays)
         for cls in self.wscontroller._ws_complex_types:
             for key in ctvalues(cls):
                 soap_type(getattr(cls, key), arrays)
