@@ -3,6 +3,8 @@ import httplib
 import cStringIO as StringIO
 import mimetools
 
+import base64
+
 from turbogears import testutil, validate
 import cherrypy
 try:
@@ -21,6 +23,7 @@ def run_soap(method, params=""):
     request = """<?xml version="1.0"?>
 <soap:Envelope
 xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"
+xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
 soap:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
 
   <soap:Body xmlns="http://foo.bar.baz/soap/">
@@ -32,6 +35,7 @@ soap:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
 </soap:Envelope>
 """ % dict(method=method, params=params)
 
+    print request
     testutil.create_request("/soap/", rfile=StringIO.StringIO(request), 
                             method="POST", 
                             headers={"Content-Length" : str(len(request)),
@@ -461,3 +465,51 @@ def test_rw_typedproperty():
 </rwp>""")
     print output
     assert '<getandsetrwpropResponse><result xsi:type="ReadWriteProperty"><value xsi:type="xsd:string">AValue</value></result></getandsetrwpropResponse>' in output
+
+
+def test_datetime():
+    cherrypy.root = DateTimeService("http://foo.bar.baz/")
+    testutil.create_request("/soap/api.wsdl")
+    output = cherrypy.response.body[0]
+    print "WSDL:\n\n"
+    print output
+    assert "xsd:dateTime" in output
+
+    output = run_soap("combine", "<d>2009-07-25</d><t>22:14:53.89654242</t>")
+    print output
+
+    assert """<combineResponse><result xsi:type="xsd:dateTime">2009-07-25T22:14:53.896542</result></combineResponse>""" in output
+
+    output = run_soap("split", "<dt>2009-07-25T22:14:53.896542</dt>")
+    print output
+    assert """<splitResponse><result xsi:type="DateTimeStruct"><d xsi:type="xsd:date">2009-07-25</d><t xsi:type="xsd:time">22:14:53.896542</t></result></splitResponse>""" in output
+
+def test_binary():
+    cherrypy.root = BinaryService("http://foo.bar.baz/")
+    testutil.create_request("/soap/api.wsdl")
+    output = cherrypy.response.body[0]
+    print "WSDL:\n\n"
+    print output
+    assert "xsd:base64Binary" in output
+
+    data = '\xc2\xc3\xc4'
+
+    output = run_soap("reverse", "<data>%s</data>" % base64.encodestring(data))
+    print output
+    
+    root = et.fromstring(output)
+    r = root.find(".//{http://foo.bar.baz/soap/types}result")
+    rdata = base64.decodestring(r.text)
+
+    assert data == rdata[::-1]
+
+def test_null():
+    ci = ComplexService("http://foo.bar.baz/")
+    cherrypy.root = ci
+    output = run_soap("getandsetrwprop", """<rwp>
+    <value xsi:nil="true"/>
+</rwp>""")
+    print output
+    assert '<getandsetrwpropResponse><result xsi:type="ReadWriteProperty"><value xsi:nil="true"/></result></getandsetrwpropResponse>' in output
+
+
